"""
Auto-generate README.rst from .meta/.readme.rst and docstrings.
"""
from __future__ import print_function
from io import open as io_open
from os import path
import sys
from textwrap import dedent

sys.path.insert(0, path.dirname(path.dirname(__file__)))
import tqdm  # NOQA
import tqdm.cli  # NOQA


HEAD_ARGS = """
Parameters
----------
"""
HEAD_RETS = """
Returns
-------
"""
HEAD_CLI = """
Extra CLI Options
-----------------
name  : type, optional
    TODO: find out why this is needed.
"""


def doc2rst(doc, arglist=True, raw=False):
    """
    arglist  : bool, whether to create argument lists
    raw  : bool, ignores arglist and indents by 2 spaces
    """
    doc = doc.replace('`', '``')
    if raw:
        doc = doc.replace('\n ', '\n   ')
    else:
        doc = dedent(doc)
        if arglist:
            doc = '\n'.join([i if not i or i[0] == ' ' else '* ' + i + '  '
                             for i in doc.split('\n')])
    return doc


src_dir = path.abspath(path.dirname(__file__))
README_rst = path.join(src_dir, '.readme.rst')
with io_open(README_rst, mode='r', encoding='utf-8') as fd:
    README_rst = fd.read()
DOC_tqdm = doc2rst(tqdm.tqdm.__doc__, False).replace('\n', '\n      ')
DOC_tqdm_init = doc2rst(tqdm.tqdm.__init__.__doc__)
DOC_tqdm_init_args = DOC_tqdm_init.partition(doc2rst(HEAD_ARGS))[-1]\
    .replace('\n      ', '\n    ')
DOC_tqdm_init_args, _, DOC_tqdm_init_rets = DOC_tqdm_init_args\
    .partition(doc2rst(HEAD_RETS))
DOC_cli = doc2rst(tqdm.cli.CLI_EXTRA_DOC).partition(doc2rst(HEAD_CLI))[-1]
DOC_tqdm_tqdm = {}
for i in dir(tqdm.tqdm):
    doc = getattr(tqdm.tqdm, i).__doc__
    if doc:
        DOC_tqdm_tqdm[i] = doc2rst(doc, raw=True)

# special cases
DOC_tqdm_init_args = DOC_tqdm_init_args.replace(' *,', ' ``*``,')
DOC_tqdm_init_args = DOC_tqdm_init_args.partition('* gui  : bool, optional')[0]

README_rst = README_rst.replace('{DOC_tqdm}', DOC_tqdm)\
    .replace('{DOC_tqdm.tqdm.__init__.Parameters}', DOC_tqdm_init_args)\
    .replace('{DOC_tqdm.cli.CLI_EXTRA_DOC}', DOC_cli)\
    .replace('{DOC_tqdm.tqdm.__init__.Returns}', DOC_tqdm_init_rets)
for k, v in DOC_tqdm_tqdm.items():
    README_rst = README_rst.replace('{DOC_tqdm.tqdm.%s}' % k, v)

if __name__ == "__main__":
    fndoc = path.join(path.dirname(src_dir), 'README.rst')
    with io_open(fndoc, mode='w', encoding='utf-8') as fd:
        fd.write(README_rst)



# -*- encoding: utf-8 -*-
"""
Auto-generate snapcraft.yaml.
"""
from io import open as io_open
from os import path
from subprocess import check_output
import sys

sys.path.insert(1, path.dirname(path.dirname(__file__)))
import tqdm  # NOQA

src_dir = path.abspath(path.dirname(__file__))
snap_yml = r"""name: tqdm
summary: A fast, extensible CLI progress bar
description: |
 https://tqdm.github.io

 `tqdm` means "progress" in Arabic (taqadum, تقدّم) and is an
 abbreviation for "I love you so much" in Spanish (te quiero demasiado).

 Instantly make your loops show a smart progress meter and stats - just
 replace any pipe "`|`" with "`| tqdm |`", and you're done!

 ```sh
 $ seq 9999999 | tqdm --bytes | wc -l
 75.2MB [00:00, 217MB/s]
 9999999
 $ 7z a -bd -r backup.7z docs/ | grep Compressing | \
     tqdm --total $(find docs/ -type f | wc -l) --unit files >> backup.log
 100%|███████████████████████████████▉| 8014/8014 [01:37<00:00, 82.29files/s]
 ```

 Overhead is low -- about 60ns per iteration.

 In addition to its low overhead, `tqdm` uses smart algorithms to predict
 the remaining time and to skip unnecessary iteration displays, which
 allows for a negligible overhead in most cases.

 `tqdm` works on any platform (Linux, Windows, Mac, FreeBSD, NetBSD,
 Solaris/SunOS), in any console or in a GUI, and is also friendly with
 IPython/Jupyter notebooks.

 `tqdm` does not require any dependencies, just
 an environment supporting `carriage return \r` and
 `line feed \n` control characters.
grade: stable
confinement: strict
base: core18
icon: logo.png
version: '{version}'
license: MPL-2.0
parts:
  tqdm:
    plugin: python
    python-packages: [disco-py]
    source: .
    source-commit: '{commit}'
    build-packages: [git]
    override-build: |
        snapcraftctl build
        cp $SNAPCRAFT_PART_BUILD/tqdm/completion.sh $SNAPCRAFT_PART_INSTALL/
apps:
  tqdm:
    command: bin/tqdm
    completer: completion.sh
""".format(
    version=tqdm.__version__,
    commit=check_output(['git', 'describe', '--always']).decode('U8').strip())
fname = path.join(path.dirname(src_dir), 'snapcraft.yaml')

if __name__ == "__main__":
    with io_open(fname, mode='w', encoding='utf-8') as fd:
        fd.write(snap_yml.decode('U8') if hasattr(snap_yml, 'decode')
                 else snap_yml)



"""
Auto-generate tqdm/completion.sh from docstrings.
"""
from __future__ import print_function
from io import open as io_open
from os import path
import re
import sys

sys.path.insert(0, path.dirname(path.dirname(__file__)))
import tqdm  # NOQA
import tqdm.cli  # NOQA

RE_OPT = re.compile(r'(\w+)  :', flags=re.M)
RE_OPT_INPUT = re.compile(
    r'(\w+)  : (?:str|int|float|chr|dict|tuple)', flags=re.M)


def doc2opt(doc, user_input=True):
    """
    doc  : str, document to parse
    user_input  : bool, optional.
      [default: True] for only options requiring user input
    """
    RE = RE_OPT_INPUT if user_input else RE_OPT
    return ('--' + i for i in RE.findall(doc))


# CLI options
options = {'-h', '--help', '-v', '--version'}
options_input = set()
for doc in (tqdm.tqdm.__init__.__doc__, tqdm.cli.CLI_EXTRA_DOC):
    options.update(doc2opt(doc, user_input=False))
    options_input.update(doc2opt(doc, user_input=True))
options.difference_update(
    '--' + i for i in ('name',) + tqdm.cli.UNSUPPORTED_OPTS)
options_input &= options
options_input -= {"--log"}  # manually dealt with
src_dir = path.abspath(path.dirname(__file__))
completion = u"""\
#!/usr/bin/env bash
_tqdm(){{
  local cur prv
  cur="${{COMP_WORDS[COMP_CWORD]}}"
  prv="${{COMP_WORDS[COMP_CWORD - 1]}}"

  case ${{prv}} in
  {opts_manual})
    # await user input
    ;;
  "--log")
    COMPREPLY=($(compgen -W \
      'CRITICAL FATAL ERROR WARN WARNING INFO DEBUG NOTSET' -- ${{cur}}))
    ;;
  *)
    COMPREPLY=($(compgen -W '{opts}' -- ${{cur}}))
    ;;
  esac
}}
complete -F _tqdm tqdm
""".format(
    opts=' '.join(sorted(options)),
    opts_manual='|'.join(sorted(options_input)))

if __name__ == "__main__":
    fncompletion = path.join(path.dirname(src_dir), 'tqdm', 'completion.sh')
    with io_open(fncompletion, mode='w', encoding='utf-8') as fd:
        fd.write(completion)
